Attribute VB_Name = "mdlGDIPlus"
Option Explicit

' ------------------------------------------------------------------
' Autor:        Leandro I. Ascierto
' Web:          www.leandroascierto.com.ar
' Fecha:        08 de Marzo de 2011
' Descripcin:  Ejecutable encargado de realizar las peticiones HTTP
'               del proyecto Facebook Photo Uploader.
' ------------------------------------------------------------------

Private Declare Function GdiplusStartup Lib "gdiplus" (Token As Long, inputbuf As GdiplusStartupInput, Optional ByVal outputbuf As Long = 0) As Long
Private Declare Function GdipDisposeImage Lib "gdiplus" (ByVal Image As Long) As Long
Private Declare Function GdipSaveImageToStream Lib "gdiplus" (ByVal Image As Long, ByVal Stream As IUnknown, clsidEncoder As GUID, encoderParams As Any) As Long
Private Declare Function GdipGetImageBounds Lib "GdiPlus.dll" (ByVal nImage As Long, srcRect As RECTF, srcUnit As Long) As Long
Private Declare Function GdipDeleteGraphics Lib "gdiplus" (ByVal graphics As Long) As Long
Private Declare Sub GdiplusShutdown Lib "gdiplus" (ByVal Token As Long)
Private Declare Function GdipDrawImageRect Lib "GdiPlus.dll" (ByVal mGraphics As Long, ByVal mImage As Long, ByVal mX As Single, ByVal mY As Single, ByVal mWidth As Single, ByVal mHeight As Single) As Long
Private Declare Function GdipGetImageGraphicsContext Lib "gdiplus" (ByVal Image As Long, ByRef graphics As Long) As Long
Private Declare Function GdipCreateBitmapFromScan0 Lib "gdiplus" (ByVal Width As Long, ByVal Height As Long, ByVal stride As Long, ByVal PixelFormat As Long, scan0 As Any, BITMAP As Long) As Long
Private Declare Function GdipGraphicsClear Lib "GdiPlus.dll" (ByVal mGraphics As Long, ByVal mColor As Long) As Long
Private Declare Function GdipCreateBitmapFromHBITMAP Lib "gdiplus" (ByVal hbm As Long, ByVal hpal As Long, ByRef BITMAP As Long) As Long
Private Declare Function GdipLoadImageFromFile Lib "GdiPlus.dll" (ByVal mFilename As Long, ByRef mImage As Long) As Long
Private Declare Function GdipSetInterpolationMode Lib "gdiplus" (ByVal graphics As Long, ByVal InterpolationMode As InterpolationMode) As Long

Private Declare Function VarPtrArray Lib "msvbvm60.dll" Alias "VarPtr" (ByRef ptr() As Any) As Long
Public Declare Sub CopyMemory Lib "kernel32.dll" Alias "RtlMoveMemory" (ByRef Destination As Any, ByRef Source As Any, ByVal Length As Long)
Private Declare Function GlobalAlloc Lib "kernel32" (ByVal uFlags As Long, ByVal dwBytes As Long) As Long
Private Declare Function GlobalLock Lib "kernel32" (ByVal hMem As Long) As Long
Private Declare Function GlobalUnlock Lib "kernel32" (ByVal hMem As Long) As Long
Private Declare Function GlobalSize Lib "kernel32" (ByVal hMem As Long) As Long
Private Declare Function GetHGlobalFromStream Lib "ole32" (ByVal ppstm As Long, hGlobal As Long) As Long
Private Declare Function CreateStreamOnHGlobal Lib "ole32" (ByVal hGlobal As Long, ByVal fDeleteOnRelease As Long, ppstm As Any) As Long
Private Declare Function CLSIDFromString Lib "ole32" (ByVal str As Long, ID As GUID) As Long

Private Type GdiplusStartupInput
    GdiPlusVersion           As Long
    DebugEventCallback       As Long
    SuppressBackgroundThread As Long
    SuppressExternalCodecs   As Long
End Type


Private Type RECTF
    nLeft           As Single
    nTop            As Single
    nWidth          As Single
    nHeight         As Single
End Type

Private Type GUID
    Data1           As Long
    Data2           As Integer
    Data3           As Integer
    Data4(0 To 7)   As Byte
End Type

Private Type EncoderParameter
    GUID            As GUID
    NumberOfValues  As Long
    Type            As Long
    Value           As Long
End Type
 
Private Type EncoderParameters
    Count           As Long
    Parameter(15)   As EncoderParameter
End Type

Private Enum InterpolationMode
    InterpolationModeInvalid = &HFFFFFFFF
    InterpolationModeDefault = &H0
    InterpolationModeLowQuality = &H1
    InterpolationModeHighQuality = &H2
    InterpolationModeBilinear = &H3
    InterpolationModeBicubic = &H4
    InterpolationModeNearestNeighbor = &H5
    InterpolationModeHighQualityBilinear = &H6
    InterpolationModeHighQualityBicubic = &H7
End Enum

Private Const ImageCodecJPG = "{557CF401-1A04-11D3-9A73-0000F81EF32E}"
Private Const EncoderQuality = "{1D5BE4B5-FA4A-452D-9CDD-5DB35105E7EB}"
Private Const EncoderParameterValueTypeLong As Long = 4
Private Const GdiPlusVersion            As Long = 1&
Private Const PixelFormat24bppRGB       As Long = &H21808
Private Const UnitPixel                 As Long = &H2&


Private Function CreateStream(byteContent() As Byte, Optional byteOffset As Long = 0&) As stdole.IUnknown
    
    ' Purpose: Create an IStream-compatible IUnknown interface containing the
    ' passed byte aray. This IUnknown interface can be passed to GDI+ functions
    ' that expect an IStream interface -- neat hack
    
    On Error GoTo HandleError
    Dim o_lngLowerBound         As Long
    Dim o_lngByteCount          As Long
    Dim o_hMem                  As Long
    Dim o_lpMem                 As Long
     
    If iparseIsArrayEmpty(VarPtrArray(byteContent)) = 0& Then ' create a growing stream as needed
         Call CreateStreamOnHGlobal(0, 1, CreateStream)
    Else                                        ' create a fixed stream
         o_lngByteCount = UBound(byteContent) - byteOffset + 1
         o_hMem = GlobalAlloc(&H2&, o_lngByteCount)
         If o_hMem <> 0 Then
             o_lpMem = GlobalLock(o_hMem)
             If o_lpMem <> 0 Then
                 CopyMemory ByVal o_lpMem, byteContent(byteOffset), o_lngByteCount
                 Call GlobalUnlock(o_hMem)
                 Call CreateStreamOnHGlobal(o_hMem, 1, CreateStream)
             End If
         End If
     End If
    
HandleError:
End Function


 Private Function ArrayFromStream(Stream As IUnknown, arrayBytes() As Byte) As Boolean

    ' Purpose: Return the array contained in an IUnknown interface
    
    Dim o_hMem As Long, o_lpMem As Long
    Dim o_lngByteCount As Long
    
    If Not Stream Is Nothing Then
    
        If GetHGlobalFromStream(ByVal ObjPtr(Stream), o_hMem) = 0 Then
            o_lngByteCount = GlobalSize(o_hMem)
            If o_lngByteCount > 0 Then
                o_lpMem = GlobalLock(o_hMem)
                If o_lpMem <> 0 Then
                    ReDim arrayBytes(0 To o_lngByteCount - 1)
                    CopyMemory arrayBytes(0), ByVal o_lpMem, o_lngByteCount
                    GlobalUnlock o_hMem
                    ArrayFromStream = True
                End If
            End If
        End If

    End If
    
End Function

Public Function iparseIsArrayEmpty(FarPointer As Long) As Long
  ' test to see if an array has been initialized
  CopyMemory iparseIsArrayEmpty, ByVal FarPointer, 4&
End Function

Public Function SaveImageToStream(ByVal hImage As Long, ByRef OutStream() As Byte, Optional ByVal JPG_Quality As Long = 85) As Boolean
    Dim IIStream    As IUnknown
    Dim tEncoder    As GUID
    Dim tParams     As EncoderParameters
    
    Erase OutStream
       
    Set IIStream = CreateStream(OutStream)

    If Not IIStream Is Nothing Then

        CLSIDFromString StrPtr(ImageCodecJPG), tEncoder

        With tParams
            .Count = 1
            .Parameter(0).NumberOfValues = 1
            .Parameter(0).Type = EncoderParameterValueTypeLong
            .Parameter(0).Value = VarPtr(JPG_Quality)
            CLSIDFromString StrPtr(EncoderQuality), .Parameter(0).GUID
        End With
        
        If GdipSaveImageToStream(hImage, IIStream, tEncoder, tParams) = 0& Then
            SaveImageToStream = ArrayFromStream(IIStream, OutStream())
        End If
        
    End If
End Function



Public Function ResizeImageToStreamJPG(ByVal sPath As String, ByVal lSize As Long, ByVal JPG_Quality As Long, OutStream() As Byte) As Boolean
    Dim GDIsi       As GdiplusStartupInput
    Dim hGraphics   As Long
    Dim gToken      As Long
    Dim hBitmap     As Long
    Dim tRect       As RECTF
    Dim hImg        As Long
    Dim PLeft       As Double
    Dim PTop        As Double
    Dim hPen        As Long
    Dim ReqWidth As Long, ReqHeight As Long
    Dim HScale As Double, VScale As Double
    Dim MyScale As Double

    GDIsi.GdiPlusVersion = GdiPlusVersion
    
    GdiplusStartup gToken, GDIsi
    
    If GdipLoadImageFromFile(StrPtr(sPath), hBitmap) = 0 Then
    
        GdipGetImageBounds hBitmap, tRect, UnitPixel
        
        If tRect.nWidth < lSize And tRect.nHeight < lSize Then
            lSize = IIf(tRect.nWidth > tRect.nHeight, tRect.nWidth, tRect.nHeight)
        End If
     
        HScale = lSize / tRect.nWidth
        VScale = lSize / tRect.nHeight
                
        MyScale = IIf(VScale >= HScale, HScale, VScale)
                
        ReqWidth = tRect.nWidth * MyScale
        ReqHeight = tRect.nHeight * MyScale
     
     
        If GdipCreateBitmapFromScan0(ReqWidth, ReqHeight, 0&, PixelFormat24bppRGB, ByVal 0&, hImg) = 0 Then

            If GdipGetImageGraphicsContext(hImg, hGraphics) = 0 Then
                
                GdipGraphicsClear hGraphics, &HFFFFFFFF
                'GdipSetInterpolationMode hGraphics, Val(Form1.Text1)
                
                Call GdipDrawImageRect(hGraphics, hBitmap, 0, 0, ReqWidth, ReqHeight)
                GdipDeleteGraphics hGraphics
                ResizeImageToStreamJPG = SaveImageToStream(hImg, OutStream, JPG_Quality)
            End If
            
            Call GdipDisposeImage(hImg)
        End If

        Call GdipDisposeImage(hBitmap)
  
    End If
    
    GdiplusShutdown gToken
End Function

